# -*- coding: utf-8 -*-
###########################################################################################################
#   Universidad de la República, Facultad de Ingeniería, Instituto de Ingeniería Eléctrica 
#
#   Copyright 2017 Universidad de la República, Facultad de Ingeniería, Instituto de Ingeniería Eléctrica 
#
#   Autores: Pablo Pérez-Nicoli, Francisco Veirano, Juan Pablo Oliver
#
#   Version: 1.0
#
#   This file is part of HTERM 3.0.
#
#   HTERM 3.0 is free software: you can redistribute it and/or modify it under the terms of the GNU 
#	 General Public License as published by the Free Software Foundation, either #version 3 of the 
#   License, or (at your option) any later version.
#
#   HTERM 3.0 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
#   even the implied warranty ofMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with HTERM 3.0. 
#   If not, see <http://www.gnu.org/licenses/>.
###########################################################################################################
from PyQt4.QtGui import QMessageBox
from PyQt4.QtGui import QPushButton
from numpy import exp
from numpy import log
from numpy import sqrt
from numpy import pi
from numpy import sinh
from numpy import cosh
from numpy import sin
from numpy import cos
from numpy import matrix
from numpy import angle
from numpy import absolute
from numpy import arange



class Calculos():
    def __init__(self,mainWind):
        # Variable de condiciones ambientales necesarias para hacer los cálculos
        # Son actualizadas por obtengoParametros.
        self.humedadExt= 90
        self.humedadInt=80
        self.resistExt=0.04
        self.resistInt=0.13
        self.tempExt=0
        self.tempInt=18
        self.deltatemp = 5
        self.resistIntCond=0.25
        
        
        # Estas variables indican que columna de la matriz muro son cada parámetro.
        self.indiceEspe=1
        self.indiceDens=2
        self.indiceMasa=3
        self.indiceCond=4
        self.indiceR=6
        self.indicePerm=8
        self.indiceZ=9
        self.indiceMu=11
        self.indiceCap=5         
        self.mainWind=mainWind
            
    def verifCamaraAire(self,muro):
        # Función que verifica que las cámaras de aire no sean ni la primera ni
        # la ultima capa del muro. Devuelve en grafico True si no estan en los
        # bordes y False si se encuentran en los bordes.
        grafico=True
        # La siguiente variable tiene los numeros de capas del muro que tienen  
        # una camara de aire muy ventilada.
        camaraAire = self.mainWind.generador_muro_obj.camara_MV
        # Me fijo si esa capa es la primera o la última 
        for i in range(len(camaraAire)):
            if (camaraAire[i]==0)or(camaraAire[i]==len(muro)-1):
                grafico=False
        # La siguiente variable tiene los numeros de capas del muro que tienen  
        # una camara de aire no muy ventilada.
        camaraAire= self.mainWind.generador_muro_obj.camara_NMV
        # Para cada una de las camaras de aire no muy ventiladas, nos fijamos
        # si estan en la primera o ultima capa del muro.
        for i in range(len(camaraAire)):
            if (camaraAire[i]==0)or(camaraAire[i]==len(muro)-1):
                grafico=False
        # Devuelve la variable booleana grafico que indica con truee 
        return grafico    
    
    
    def calcTemp(self,muro,rsi,rse,ti,te):
    # Calculo temperaturas del muro en cada capa. Devuelve un vector con las 
    # temperaturas en cada una de las capas.
            
    # El cálculo es bien simple porque es lineal. Es un divisor resistivo.
    # Primero calculamos la resistencia total y despues vamos viendo con 
    # las temperaturas del exterior e interior, como quedan las tempertauras 
    # (voltajes) en las capas intermedias. ti+1 = ti - (Ri/Rtot)(tint-text) 
    # para ver la tem en cada capa i.
            
        # Calculamos la resistencia termica total. Incluye las resistencias 
        # superficiales exterior e interior. 
        rtot = rsi+rse
        for i in range(len(muro)): rtot = rtot+muro[i][self.indiceR]                        
    
        # Primer temperatura es la interna
        temp = [ti]
        # La segunda es después de la resistencia superficial interior
        taux = temp[0] - (rsi/rtot)*(ti-te)
        temp.append(taux)        
        
        # Ahora se calcula para cada capa del muro 
        for i in range(len(muro)):
            taux = temp[-1] - (muro[i][self.indiceR]/rtot)*(ti-te)
            temp.append(taux)       
        # Al final se agrega la temperatura exterior
        temp.append(te)
        return temp # Devuelve la lista de temperaturas
    
    def calcTrans(self,muro ,rsi,rse):
    # Calculo de transmitancia, incluye las resistencias superficiales externas
    # e internas.
        rtot = rse+rsi
        for i in range(len(muro)): rtot = rtot+muro[i][self.indiceR]
        U = 1/rtot
        return U
    
    
    def calcTempRoc(self,muro,hri,hre,ti,te):
    # Calculo temperaturas de rocio. Es también lineal pero en las presiones de vapor
    # pi+1 = pi - (rvap_i/rvaptot)*(pint-pext).
    
        # Las presiones externas e internas se calaculan a partir de las temperaturas 
        # y las humedades externas e internas.              
        pint=self.getPres(ti)*hri/100
        pext=self.getPres(te)*hre/100
        pres = [pint]
        
        # Hay que hacer una diferencia con los materiales que tienen una permeabilidad 
        # (indiceMU>1e6), estos en realidad son infinitamente resistentes al pasaje de 
        # vapor entonces si hay uno de esos materiales, se considera que toda la temp de 
        # rocio cae ahí, entonces a partir de ahi se pone la presion exterior en cada
        # capa.                     
        hay_metal=False # bandera que se prende cuando aparece un de estos materiales
        rtotvap= 0
        
        # Calculamos la rtotvap hasta el primer material con mu>1e6
        for i in range(len(muro)): 
                rtotvap = rtotvap+muro[i][self.indiceZ]
                
        hay_metal=False # Bajamos la bandera 
        # Hasta que se encuentra ese material se van calaculando las presiones
        # de manera lineal. A partir de que aparece el material se ponen todas las
        # capas como presion exterior
        for i in range(len(muro)-1):
            if (muro[i][self.indiceMu]>=1e6)&(muro[i][self.indiceMu]!=u'') : hay_metal=True
            if hay_metal==False:       
                presaux = pres[-1] - (muro[i][self.indiceZ]/rtotvap)*(pint-pext) 
                pres.append(presaux)
                print pres
            else:
                pres.append(pext)    
                    
        pres.append(pext)
        
        # se utiliza la función gettemvap para pasar de presiones a temperaturas
        # rocio.
        tempVap=[]
        for i in range(len(pres)): tempVap.append(self.getTempVap(pres[i]))
        
        # devuelve la lista de temperaturas de rocio
        return tempVap
    
    def getPres(self,ts):
        # Implementa el pasaje de temperatura de rocio a presion de vapor
        # el hterm viejo lo hacia a partir de una tabla
        if ts>=0: ps = 610.5*exp(17.269*ts/(237.3+ts))
        if ts<0: ps = 610.5*exp(21.875*ts/(265.5+ts))
        return ps
    def getTempVap(self,ps):
        # Implementa el pasaje de presion de vapor a temp de rocio
        if ps>=610.5: t =log(ps/610.5)*237.3/(17.269-log(ps/610.5))
        if ps<610.5: t = log(ps/610.5)*265.5/(21.875-log(ps/610.5))
        return t    
    
    def calcCond(self, muro):
    # Función encargada de calcular la temperatura  y la temperatura de rocio
    # en cada capa del muro. Recibe como parámetros una variable muro.
    # Devuelve temp,tempVap,espesores,M,U,espetot        
        
        # Nos fijamos si el muro tiene camaras de aire muy ventiladas. Si tiene nos quedamos solo
        # el indice de la primera que aprezca empezando desde el interior. 
        camaraAire= self.mainWind.generador_muro_obj.camara_MV # Indices donde se encuentran cada capa que es camara de aire muy ventilada  
        # Para que sea más facil paso la lista a un numero y none si esta vacio.
        if camaraAire==[]: camaraAire=None 
        else: camaraAire=camaraAire[0]      
    
        # Diferenciamos entre si hay camara de aire muy ventilada o no.
        if camaraAire==None:
            # Si no hay se calculan las temp y las temp de rocio para cada capa y la transmitancia total
            temp = self.calcTemp(muro,self.resistIntCond,self.resistExt,self.tempInt,self.tempExt)
            tempVap=self.calcTempRoc(muro,self.humedadInt,self.humedadExt,self.tempInt,self.tempExt)                
            U=self.calcTrans(muro,self.resistInt,self.resistExt)
        else:                                        
            # Si hay camara de aire me quedo con el muro hasta la camara de aire
            muro_aux=muro[0:camaraAire][:] # Esto toma hasta camaraire sin incluirla
            # Calculo las temperaturas de cada capa
            temp = self.calcTemp(muro_aux,self.resistIntCond,self.resistIntCond,self.tempInt,self.tempExt)
            # La penultima temperatura corresponde con la que caería en una resistencia superficial de
            # la camara de aire. Pero no la estamos graficando y por eso la eliminamos
            del temp[-2]
            # Le agregamos temperaturas exterior para el resto de las capas que vengan despues de 
            # la camara de aire. 
            for i in range(len(muro)-camaraAire+1): temp.append(temp[-1])
            # Calculamos las temperaturas de vapor para cada capa hasta la camara de aire
            tempVap=self.calcTempRoc(muro_aux,self.humedadInt,self.humedadExt,self.tempInt,self.tempExt)
            # le asignamos la temp de vapor externa a las capas que vienen despues
            for i in range(len(muro)-camaraAire): tempVap.append(tempVap[-1])
            # Calculamos la transmitancia hasta la camara de aire
            U=self.calcTrans(muro_aux,self.resistInt,self.resistExt)  

            # Esta es otra opción que es que no dibuje el muro de la cpaa de aire en adelante
            muro=muro[0:camaraAire][:]   
            temp = self.calcTemp(muro,self.resistIntCond,self.resistIntCond,self.tempInt,self.tempExt)
            tempVap=self.calcTempRoc(muro,self.humedadInt,self.humedadExt,self.tempInt,self.tempExt)                
            U=self.calcTrans(muro,self.resistInt,self.resistExt)
    
            
        # Generamos vector de espesores y calculamos masa total y espesortotal      
        espesores=[]
        M = 0
        espetot = 0           
        for i in range(len(muro)): 
            espesores.append(muro[i][self.indiceEspe])
            if muro[i][self.indiceMasa]!='':  M = M+muro[i][self.indiceMasa]            
            espetot = espetot+muro[i][self.indiceEspe]              
        
        return temp,tempVap,espesores,M,U,espetot



    def calcAmor(self,muro):
    # Esta función calcula el factor de amortiguacion y el retardo termico
            
        # Nos fijamos si el muro tiene camaras de aire muy ventiladas. Si tiene nos quedamos solo
        # el indice de la primera que aprezca empezando desde el interior. 
        camaraAire= self.mainWind.generador_muro_obj.camara_MV # Indices donde se encuentran cada capa que es camara de aire muy ventilada  
        # Para que sea más facil paso la lista a un numero y none si esta vacio.
        if camaraAire==[]: camaraAire=None 
        else: camaraAire=camaraAire[0]
        
        camaraAireNoVent = self.mainWind.generador_muro_obj.camara_NMV

        rsi = self.resistInt
        rse = self.resistExt
        muroaux=muro[:][:]
        # Si hay camara de aire trunco el muro antes de la camara y le asigno a la
        # resistencia superficial externa la interna (es así por norma)
        if (camaraAire!=None): 
            muroaux=muro[0:camaraAire][:]
            rse =rsi
        # Me fijo si todas las capas tiene capacidad calorifica sino no puedo graficar 
        nografico=False     
        for i in range(len(muroaux)):  
            if (muroaux[i][self.indiceCap]==u'None')|(muroaux[i][self.indiceCap]==u'nan')|(muroaux[i][self.indiceCap]==u''):
                nografico=True
                        
        if nografico==False:   
        # Si hay cpacidad calorica, calculo basandme en laa noma 13786.
            T = 86400   # Este es el período en segundos 24*60*60    
            # Hay que calcular ciertos parametros que los indica la norma
            penetvec =[]
            sedavec = []
            # y con esos parametros se calculan las matrices de la norma que 
            # relacionan el flujo de calor y temperatura en la capa exterior con
            # el flujo de calor y la temperatura de la capa interiror.
            z11vec=[]
            z12vec= []
            z21vec= []
            
            for i in range(len(muroaux)):  
            # Primero hay que calcular la matriz de cada capa y luego multiplicarlas
                # Primero me 
                capaire=False
                for l in range(len(camaraAireNoVent)):
                    if i==camaraAireNoVent[l]: capaire=True
                    
                # Aca hay que diferenciar entre cuatro casos ya que la matriz de capa depende si es 
                # una capa sin resistencia termica, si no tiene masa, si es una camara de aire
                # poco ventilada o si es un material normal. 
                if muroaux[i][self.indiceR]==0:
                # El caso sin resistencia termica se toma como que no afecta la amortiguacion y
                # por lo tanto se usa la matriz identidad.
                    z11=1
                    z11vec.append(z11)
                    z12=0
                    z12vec.append(z12)              
                    z21=0
                    z21vec.append(z21)
                elif (muroaux[i][self.indiceMasa]==0)|(muroaux[i][self.indiceMasa]==u'None')|(muroaux[i][self.indiceMasa]==u'nan')|(muroaux[i][self.indiceDens]==u''):
                # En el caso de que no haya masa se usa la siguiente matriz, es la misma que para 
                # una cámara de aire poco ventilada que indica la norma.
                    z11=1
                    z11vec.append(z11)
                    z12=-muroaux[i][self.indiceR]
                    z12vec.append(z12)              
                    z21=0
                    z21vec.append(z21)
                elif capaire==False:  
                # En el caso de materiales "comunes" se usa la matriz indicada en la norma    
                    # Estas las recalculo porque no se me garantiza que vengan en el muro. 
                    # Adentro de este if si me garantizan espesor, resistencia y masa .
                    conductividad = muroaux[i][self.indiceEspe]/muroaux[i][self.indiceR]
                    densidad = muroaux[i][self.indiceMasa]/muroaux[i][self.indiceEspe]    
                    penet=(sqrt(conductividad*T/(pi*densidad*muroaux[i][self.indiceCap])))
                    penetvec.append(penet)            
                    seda=(muroaux[i][self.indiceEspe]/penet)
                    sedavec.append(seda)
                    z11=cosh(seda)*cos(seda)+1j*sinh(seda)*sin(seda)
                    z11vec.append(z11)
                    z12=(-penet/(2*conductividad))*(sinh(seda)*cos(seda)+cosh(seda)*sin(seda)+1j*(cosh(seda)*sin(seda)-sinh(seda)*cos(seda)))
                    z12vec.append(z12)              
                    z21=(-conductividad/penet)*(sinh(seda)*cos(seda)-cosh(seda)*sin(seda)+1j*(cosh(seda)*sin(seda)+sinh(seda)*cos(seda)))
                    z21vec.append(z21)                    
                else:
                # Este es el caso de la camara de aire poco ventilada. En realidad nunca llega 
                # hasta aca porque la camra de aire poco ventilada no tiene masa
                # pero se hace lo mismo
                    z11=1
                    z11vec.append(z11)
                    z12=-muroaux[i][self.indiceR]
                    z12vec.append(z12)              
                    z21=0
                    z21vec.append(z21)                
            # Se realiza la multiplicaci´on de cada una de las matrices. El elemento z22 es siempre
            # igual al z11.
            zaux = matrix(((z11vec[0],z12vec[0]),(z21vec[0],z11vec[0])))
            if  len(muroaux)>=2:
                for i in range(len(muroaux)-1):       
                    zi = matrix(((z11vec[i+1],z12vec[i+1]),(z21vec[i],z11vec[i+1])))            
                    zaux = zi*zaux
                             
            # Las capas superficiales tienen sus propias matrices:
            zsi = matrix(((1,-rsi),(0,1)))        
            zse = matrix(((1,-rse),(0,1)))
            # Se multiplican para obtener la matriz total de trasnferencia
            ztot = zse*zaux*zsi
            # Hasta acá es todo como indica la norma. Ahora para calcular el factor de 
            # amortiguacion y el retardo termico es necesario tomar una hipotesis extra
            # Lo que se tiene es:
            #   te = (z11 z12)  ti
            #   qe = (z21 z22)  qi
            # Se toma un seno para la temperatura te con una amplitud determinada
            # Es necesario realizar una hipotesis mas para poder determinar la ti
            # Paraeso se asume la ti=cte (0 porque estamos hablando de variaciones)
            # y se calcula la temperatura en e borde sin considerar la resistencia superficial
            # todas las variaciones son amortiguadas al final por esa resistencia superficial
            # Entonces con ztot y ti=0 se obtiene qi y qe y despues se calcula con te y qe la 
            #temperatura en la capa anterior al interior (por eso se usa ztotsi)
            ztotsi = zse*zaux
            term1 = (1/ztotsi[0,1]) - (ztot[1,1]/(ztotsi[1,1]*ztot[0,1])) 
            term2 = (ztotsi[0,0]/ztotsi[0,1])-(ztotsi[1,0]/ztotsi[1,1])
            am = term1/term2
            modamor = absolute(am)
            faseamor = angle(am)
            # Paso la fase a horas            
            faseamormost= -faseamor*T/(2*pi*60*60)    
            # si es negativa le sumo 24hs            
            if faseamormost<0 :  faseamormost=faseamormost+24                      
            #Acomodo vector tiempo para que este en horas
            tiempo = arange(0,T/(60*60),0.1)
            tiempo= tiempo.tolist() 
            varte =self.deltatemp
    
            tiempo = [x*(60*60) for x in tiempo]
            tempext = [varte*sin((2*pi/T)*x) for x in tiempo]
            tempint = [varte*(modamor)*sin(((2*pi/T)*x)+faseamor) for x in tiempo]
            tiempo = [x/(60*60) for x in tiempo]
        else:
        # Si alguna capa no tiene capacidad calorifica entonces se despliega el mensaje correspondiente     
            msg = QMessageBox() #genero la ventana
            msg.setWindowTitle(u'Error')
            msg.setIcon(QMessageBox.Warning) #le pongo el signo de warning (puedo poner alerta y pregunta)
            msg.setText(u'Faltan datos de capacidad calorífica.') # mensaje principal del error    
            msg.addButton(QPushButton('Aceptar'), QMessageBox.AcceptRole)
            msg.exec_() # No se bien que hace pero es necesario     
            tiempo=[]
            tempext=[]
            tempint=[]
            modamor=[]
            faseamormost=[]
            
        return tiempo, tempext, tempint, nografico, modamor, faseamormost   
    
