#!/usr/bin/env python2
# -*- coding: utf-8 -*-
###########################################################################################################
#   Universidad de la República, Facultad de Ingeniería, Instituto de Ingeniería Eléctrica 
#
#   Copyright 2017 Universidad de la República, Facultad de Ingeniería, Instituto de Ingeniería Eléctrica 
#
#   Autores: Pablo Pérez-Nicoli, Francisco Veirano, Juan Pablo Oliver
#
#   Version: 1.0
#
#   This file is part of HTERM 3.0.
#
#   HTERM 3.0 is free software: you can redistribute it and/or modify it under the terms of the GNU 
#	 General Public License as published by the Free Software Foundation, either #version 3 of the 
#   License, or (at your option) any later version.
#
#   HTERM 3.0 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
#   even the implied warranty ofMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with HTERM 3.0. 
#   If not, see <http://www.gnu.org/licenses/>.
###########################################################################################################
from os import getcwd
from os import remove
from fpdf import FPDF, HTMLMixin
from PyQt4.QtGui import QFileDialog
from PyQt4.QtGui import QMessageBox
from PyQt4.QtGui import QPushButton
from os import path as ph

#import html as HTML
#from matplotlib.backends.backend_pdf import PdfPages 

class PDF(FPDF, HTMLMixin): 
    def header(self):
        #Funcion que genera el titulo
        # Arial bold 15
        self.set_font('Arial', 'B', 15)
        # Calculate width of title and position
        title = 'Reporte Hterm' #titulo principal del pdf
        w = self.get_string_width(title) + 6
        self.set_x((210 - w) / 2)
        # Colors of frame, background and text
        self.set_draw_color(79, 195, 234)
        self.set_fill_color(79, 195, 234) #fondo del titulo recuadrado
        self.set_text_color(0, 0, 0) #color del texto del titulo
        # Thickness of frame (1 mm)
        self.set_line_width(1)
        # Title
        self.cell(w, 9, title, 1, 1, 'C', 1) #incerto el titulo
        # Line break
        self.ln(10)

    def footer(self):
        #Funcion que genera el pie de pagina, Solo pongo el numero de pagina.
        # Position at 1.5 cm from bottom
        self.set_y(-15)
        # Arial italic 8
        self.set_font('Arial', 'I', 8)
        # Text color in gray
        self.set_text_color(128)
        # Page number
        self.cell(0, 10, u'Página ' + str(self.page_no()), 0, 0, 'C')

    def chapter_title(self, num, label):
        #Funcion que genera el titulo de capitulo
        # Arial 12
        self.set_font('Arial', '', 12)
        # Background color
        self.set_fill_color(79, 195, 234)
        # Title
        self.cell(0, 6, u'Sección %d : %s' % (num, label), 0, 1, 'L', 1)
        # Line break
        self.ln(4)

    def chapter_body(self, txt):
        #Funcion que incerta texto en el cuerpo del capitulo.
        # Read text file
        self.set_font('Times', '', 8)
        # Output justified text
        self.multi_cell(0, 5, txt)
        # Line break
        self.ln()


class generadorPDF():
    def __init__(self,mainWind):
        self.mainWind = mainWind #variable de clase que permite acceder a todos los objetos del mainwindows
        self.mainWind.exportar_resultados.triggered.connect(self.exportar_resultados_triggered) #Se vincula la funcion principal de esta clase con el boton exportar resultados del menu archivo
        
    def exportar_resultados_triggered(self):
        if self.mainWind.generador_muro_obj.muro_valido == True: #Si el muro es valido
            self.mainWind.muestroDatosObj.actualizoDatos() #Actualizo las graficas
            cwd=getcwd() #Obtengo el current working directory
            dialogo = QFileDialog() #creo un dialogo
            dialogo.setWindowTitle('Exportar resultados') #titulo del dialogo
            dialogo.setDirectory(cwd) #el dialogo se abre en el directorio donde esta el ejecutable
            dialogo.setAcceptMode(QFileDialog.AcceptSave)
            dialogo.setNameFilter('PDF (*.pdf)') #solo muestro archivos con extension .pdf
            dialogo.setDefaultSuffix('pdf') #le impongo la extension .pdf a lo que voy a guardar
            if dialogo.exec_() == QFileDialog.Accepted:
                fname=dialogo.selectedFiles()[0]#si el usuario concreta la accion, se guarda en fname la ruta del archivo ha crear
            else:
                fname=u''#si se cancela la accion o se cierra el dialogo, se impone en fname el unicode vacio.
            if not fname==u'':#Si se selecciono un nombre de archivo a crear.
                self.genera_pdf(fname)#Se llama a la funcion que finalmente genera el pdf (la mas compleja) y se le pasa como parametro la ruta y nombre donde debe crear el mismo. Ya no es necesario preguntarle mas nada al usuario, solo se debe exportar los resultados
        else: #Si el muro no es valido se debe dar el mensaje de error
            msg = QMessageBox() #gerero la ventana
            msg.setWindowTitle('Error')
            msg.setIcon(QMessageBox.Warning) #le pongo el signo de warning (puedo poner alerta y pregunta)
            msg.setText(u'El muro no es válido, por lo tanto no es posible exportar los resultados.') # mensaje principal del error
            msg.addButton(QPushButton('Aceptar'), QMessageBox.AcceptRole)            
            retval = msg.exec_() # Se ejecuta el dialogo
            
    def genera_pdf(self,fname):
        #Funcion que se ejecuta cuando todo lo necesario para exportar los resultados ya esta listo.
        #Esta funcion genera el pdf en la ruta y nombre indicada en fname.
        pdf = PDF() #clase definida en este archivo
        self.mainWind.muestroDatosObj.fig.savefig('Figura1_tmp.png')#archivo temporal donde se guarda la figura de temperatura y temperatura de rocio del muro.
        self.mainWind.muestroDatosObj.figamor.savefig('Figura2_tmp.png')#archivo temporal donde se guarda la figura de amortiguamiento del muro.
        pdf.add_page()#se crea la primer pagina del pdf
         
        ####CAPITULO, DESCRIPCION DEL MURO####       
        pdf.chapter_title(1,u'Datos Muro')#se crea el capitulo uno, que describe el muro.
        #Se crea el primer contenido de la primera seccion. Aqui se define la nomeclaruta utilizada en la tabla que presenta al muro        
        legenda=u"""
        e  -> Espesor [mm]
        ro -> Densidad [kg/m3]
        M -> Masa [Kg/m2]
        Lambda -> Conductividad térmica [W/(m.K)]
        Cp -> Calor específico [kJ/m2.K]
        R -> Resistencia térmica [m2.K/W]
        CT -> Capacidad térmica media [kJ/(m2.K))]
        delta -> Permeabilidad al vapor de agua [kg/m.s.Pa]
        Z -> Resistencia al vapor de agua [m2.s.Pa/kg]        
        1/Z -> Permeancia al vapor de agua [kg/m2.s.Pa]   
        mu -> Factor de resistencia al vapor de agua
        Sd -> Espesor de aire equivalente Sd [m]
        OBS -> Observaciones: 
                BDO: Material proveniente de la base de datos original.
        """
        pdf.chapter_body(legenda)
        #Se comienza a crear la tabla que describe al muro. Esto se escribe en html, luego se carga al pdf con la funcion write_html de la clase FPDF
        titulos_cortos_en_orden=['e','ro', 'M','Lambda', 'Cp', 'R', 'CT' ,'delta', 'Z', '1/Z', 'mu', 'Sd','OBS'] #Headers de la tabla
        inicio_font_size= '<font size="7">' #Tamaño de letra de tabla
        encabezado='<table border="1" align="center" width="100%">' #descripcion parametros tabla
        headers ='<thead><tr>'
        ancho=99/(len(titulos_cortos_en_orden)+2) #ancho de las columnas
        headers = headers+'<th width="'+unicode(3*ancho)+'%">'+u' '+'</th>' #genera la primer columna, que no tiene header y es mas ancha porque es donde va el nombre del material
        for i in range(len(titulos_cortos_en_orden)):
            headers=headers+'<th width="'+unicode(ancho)+'%">' +titulos_cortos_en_orden[i]+ '</th>' #aca se agregan el resto de las columnas, con un ancho definido en ancho y el header definido en titulos_cortos_en_orden en orden
        headers = headers + '</tr></thead>' #codigo html
        iniciocuerpo = '<tbody>'#codigo html
        filas=self.mainWind.muro_armado.rowCount()#consigo cantidad de filas
        contenido_tabla=u'' #variable donde voy a ir agregando separado por salto de linea cada fila (cada capa)
        for i in range(filas): #para cada capa
            fila_contenido='<tr>'#codigo html
            for j in range(len(self.mainWind.generador_muro_obj.datos_muro_lista[i])):#para cada parametro
                if not(j==0 or j==(len(self.mainWind.generador_muro_obj.datos_muro_lista[i])-1)):#salvo para el primer parametro (nombre) y ultimo parametro (observacion: si es BDO o no)
                    auxiliar=self.mainWind.generador_muro_obj.formato_numeros(unicode(self.mainWind.muro_armado.item(i,j).text()))#obtengo el valor del parametro
                else:#en el caso de la primer o ultima columna
                    if (len(unicode(self.mainWind.muro_armado.item(i,j).text()))>28): #Si el largo supero los 28 caracteres (definicion arbitraria basado en espacio disponible)
                        auxiliar = unicode(self.mainWind.muro_armado.item(i,j).text())[:28]+'..'#se trunca y se agreguan dos puntos suspensivos
                    else:
                        auxiliar = unicode(self.mainWind.muro_armado.item(i,j).text())#si no es demasiado largo, lo cargo como esta. Notar que en este caso no se formatea con estilo numercio, ya que es texto.
                if len(auxiliar)==0:
                    auxiliar=u' ' #en caso que el parametro nombre o observacion esten vacios, se carga en auxiliar el unicode espacio
                #Hasta aca se logra tener en auxiliar el texto en formato deseado a poner en la columna.
                fila_contenido=fila_contenido+'<td>'+auxiliar+ '</td>'#Cargo efectivamente el texto en la fila.
            fila_contenido=fila_contenido+'</tr>'#codigo html
            contenido_tabla='\n'.join([contenido_tabla,fila_contenido])#agergo en la variable ocntenido tabla, la linea correspondiente a este material separado por un salto de linea

        fin1='</tbody>' #codigo html
        fin2='</table>' #codigo html
        fin_font_size='</font>' #codigo html
        html_3='\n'.join([inicio_font_size, encabezado, headers,iniciocuerpo, contenido_tabla,fin1, fin2, fin_font_size]) #cargo en esta variable auxiliar todas las lineas de codigo html correspondientes a la creacion de la tabla que describe el muro

        pdf.write_html(html_3) #esta linea finalmente escribe en el pdf la tabla utilizado codigo html
        
        ####CAPITULO CONDICIONES BASE####
        pdf.chapter_title(2,u'Condiciones base') #se crea el capitulo dos, que describe las condiciones base
        #En legenda se carga la nomeclatura utilizada en esta seccion
        legenda=u"""
        te  -> Temperatura Exterior [ºC]
        ti -> Temperatura Interior [ºC]
        Hre -> Humedad relativa exterior [%]
        Hri -> Humedad relativa exterior [%]
        Rse -> Resistencia superficial exterior [m2.K/W]
        Rsi -> Resistencia superficial interior [m2.K/W]
        """
        pdf.chapter_body(legenda) #inserto la legenda utilizando la funcion de la clase definida en este archivo chapter_body
        inicio_font_size= '<font size="8">'#codigo html
        encabezado='<table border="1" align="center" width="100%">'#codigo html, propiedades tabla
        headers ='<thead><tr>'#codigo html
        a_mostrar = ['te','ti','Hre','Hri','Rse','Rsi'] #nombres de headers de la tabla a generar que describe las condiciones base
        contenido = [self.mainWind.tempExt.text(),self.mainWind.tempInt.text(),self.mainWind.humedadExt.text(),self.mainWind.humedadInt.text(),self.mainWind.resistExt.text(),self.mainWind.resistIntCond.text()] #valor de cada header. Esta es una tabla de una fila (fila headers + fila valores)
        ancho=99/(len(a_mostrar)) #ancho de columnas (mismo apra todas)
        for texto in a_mostrar: #para cada columna
            headers=headers+'<th width="'+unicode(ancho)+'%">' + texto + '</th>'#genero linea html de cada columna
        headers = headers + '</tr></thead>' #codigo html
        iniciocuerpo = '<tbody>'#codigo html
        fila_contenido='<tr>' #en esa variable se ira agregando cada columna
        #ahora se carga el contenido
        for j in contenido:
            if (len(j)>21):
                auxiliar = unicode(j)[:21]+'..' #Si en cualquier caso el contenido supera los 21 caracteres se trunca a 21 caracteres y se agregan dos puntos suspensivos. 21 fue tomado arvitrariamente basado en restricciones de espacio.
            else:
                auxiliar = unicode(j) #si no supera los 21 caracteres se escribe completo
            if len(auxiliar)==0:
                auxiliar=u' '#en caso que el parametro nombre o observacion esten vacios, se carga en auxiliar el unicode espacio
            fila_contenido=fila_contenido+'<td>'+auxiliar+ '</td>' 
        fila_contenido=fila_contenido+'</tr>'

        fin1='</tbody>' #codigo html
        fin2='</table>' #codigo html
        fin_font_size='</font>' #codigo html
        
        html_4='\n'.join([inicio_font_size, encabezado, headers,iniciocuerpo, fila_contenido,fin1, fin2, fin_font_size]) #en esta variable auxiliar se junta todo el codigo html necesario para generar la tabla que presenta las condiciones base
        pdf.write_html(html_4)#esta linea finalmente escribe en el pdf la tabla utilizado codigo html
        #se agregan notas pedidas por el cuerpo de Arquitectos
        tipo_cerramiento= unicode(u'Tipo de cerramiento: ' + self.mainWind.cerramientoBox.currentText())#Se indica si es un cerramiento vertical o horizontal
        zona = unicode(self.mainWind.zonasBox.currentText()) #Se indica la zona climatica
        Comentario_sobre_datos= unicode(self.mainWind.norma.text()) #Se indica si las condiciones base estan bajo la norma o no
        texto='\n'.join([tipo_cerramiento, zona, Comentario_sobre_datos])#se une todos los textos anteriores separados por saltos de linea
        pdf.chapter_body(texto) #se incerta el texto utilizado la funcion de la clase pdf definida en este archivo
        
        y_actual= pdf.get_y() #se obtiene la posicion y actual para tomar la decicion si insertar o no un salto de pagina
        if y_actual>150: #si la pagina esta proxima a terminarse, esto dependera de la cantidad de capas del muro
            pdf.add_page() #se inserta un salto de pagina.

        ####CAPITULO GRAFICA DE CONDENSACION####
        pdf.chapter_title(3,u'Gráfica Condensación')#titulo de capitulo
        #primero se crea la tabla que indica la temperatura y temperatura de rocio en cada borde de capa.
        titulos_cortos_en_orden=['Plano' , u'Temperatura [ºC]', u'Temperatura  rocío [ºC]'] #headers de tabla que indica parametros de bordes.
        inicio_font_size= '<font size="8">' #tamaaño de letra de tabla
        encabezado='<table border="1" align="left" width="100%">' #codigo html, parametros tabla.
        headers ='<thead><tr>' #codigo html
        ancho=[50/(len(titulos_cortos_en_orden)*2), 50/(len(titulos_cortos_en_orden)), 50/(len(titulos_cortos_en_orden))] #ancho de cada columna de la tabla
        for i in range(len(titulos_cortos_en_orden)): #para cada columna
            headers=headers+'<th width="'+unicode(ancho[i])+'%">' +titulos_cortos_en_orden[i]+ '</th>'#genero su header
        headers = headers + '</tr></thead>'#codigo html
        iniciocuerpo = '<tbody>'#codigo html
        filas=self.mainWind.tabla.rowCount() #obtengo cantidad de filas
        contenido_tabla=u'' #en esta variable se ira cargando con salto de linea cada capa.
        for i in range(filas): #para cada fila
            fila_contenido='<tr>'#codigo html
            for j in range(len(titulos_cortos_en_orden)):#para cada columna
                if (len(unicode(self.mainWind.tabla.item(i,j).text()))>21): #si el contenido es mayor a 21 caracteres
                    auxiliar = unicode(self.mainWind.tabla.item(i,j).text())[:21]+'..'#trunco el contenido a 21 caracteres y agrego dos puntos suspensivos
                else:
                    auxiliar = unicode(self.mainWind.tabla.item(i,j).text()) #en otro caso el texto queda identico
                if len(auxiliar)==0:
                    auxiliar=u' '#si el contenido a cargar es vacio, se impone el unicode espacio
                #aca se tiene ne auxiliar el contenido de la celda (para cada fila para cada columna) con formato deseado
                fila_contenido=fila_contenido+'<td>'+auxiliar+ '</td>' #genero el codigo html correspondiente a la celda
            fila_contenido=fila_contenido+'</tr>'#codigo html, termino la fila
            contenido_tabla='\n'.join([contenido_tabla,fila_contenido]) #agrego la fila a la bariable contenido_tabla, cada fila separado con slato de linea 

        fin1='</tbody>'#codigo html
        fin2='</table>'#codigo html
        fin_font_size='</font>'#codigo html
        html_5='\n'.join([inicio_font_size, encabezado, headers,iniciocuerpo, contenido_tabla,fin1, fin2, fin_font_size])#uno en esta variable auxiliar todo el codigo html necesario para generar la tabla que indica las temperaturas en los bordes
        
        y_actual= pdf.get_y() #si se esta proximo al fin de la pagina se incerta un salto de pagina. Esto dependera de la cantidad de capas
        pdf.write_html(html_5)#esta linea finalmente escribe en el pdf la tabla utilizado codigo html
        #Finalmente se agrega indormacion relevante del muro como ser:
        T=unicode(self.mainWind.trans.text()+' @ Rsi='+ self.mainWind.resistInt.text()+ ' m2.K/W')#Transmitancia termica
        M=unicode(self.mainWind.masa.text())#Masa total
        E=unicode(self.mainWind.espesor.text())#espesor total
        texto='\n'.join([T, M, E]) #se unen las lineas anteriores separadas con salto de linea
        pdf.chapter_body(texto) #se inserta el texto utilizando la funcion definida en la clase definida en este archivo.
        
        y_luego_tabla=pdf.get_y() #Se obtiene la nueva posicion luego de terminada la tabla. Esto se utiliza para hubicar la figura de perfil del muro.
        if ((-y_actual+y_luego_tabla)<100):
            ancho = 100-(-y_actual+y_luego_tabla)
        else:
            ancho=0
        #ahora se inserta la figura que se habia creado de forma temporal en el current working directory
        def resource_path(relative_path): #esta funcion es necesaria para obtenera la figura temporal que se crea del perfil del muro, aun cuando se genera el exe en windows
            """ Get absolute path to resource, works for dev and for PyInstaller """
            try:
                # PyInstaller creates a temp folder and stores path in _MEIPASS
                base_path = sys._MEIPASS
            except Exception:
                base_path = ph.abspath(".")
            return ph.join(base_path, relative_path)
        securitySearchForm = resource_path("Figura1_tmp.png")
        pdf.image(securitySearchForm, 90, (y_actual+10), 110)
        pdf.cell(0.1, ancho,' ', 0, 1, 'L', 0)
        y_actual= pdf.get_y()
        if y_actual>150:
            pdf.add_page()
        
        ####CAPITULO GRAFICA DE AMORTIGUAIENTO####
        pdf.chapter_title(4,u'Gráfica Amortiguamiento')#titulo del capitulo
        y_actual= pdf.get_y()#se obtiene posicion actual de y
        A=unicode(self.mainWind.amortiguamiento.text())#se obtiene factor de amortiguamiento
        F=unicode(self.mainWind.fase.text())#se obtiene retardo termico
        texto='\n'.join([A,F])#se unen ambas lineas separadas por salto de linea
        pdf.chapter_body(texto)#se inserta la informacion anterior en el texto utilizando la funcion de la clase pdf definida en este archivo
        #Ahora se inserta la figura que se habia generado temporalmente en el current working directory sobre el amortiguamiento
        securitySearchForm = resource_path("Figura2_tmp.png")
        pdf.image(securitySearchForm, 60, y_actual, 100)
        
        pdf.output(fname, 'F')#finalmente se genera el pdf.
        remove('Figura1_tmp.png') #se borra la figura temporal del perfil del muro
        remove('Figura2_tmp.png') #se borra la figura temporal del amortiguamiento del muro.
