# -*- coding: utf-8 -*-
###########################################################################################################
#   Universidad de la República, Facultad de Ingeniería, Instituto de Ingeniería Eléctrica 
#
#   Copyright 2017 Universidad de la República, Facultad de Ingeniería, Instituto de Ingeniería Eléctrica 
#
#   Autores: Pablo Pérez-Nicoli, Francisco Veirano, Juan Pablo Oliver
#
#   Version: 1.0
#
#   This file is part of HTERM 3.0.
#
#   HTERM 3.0 is free software: you can redistribute it and/or modify it under the terms of the GNU 
#	 General Public License as published by the Free Software Foundation, either #version 3 of the 
#   License, or (at your option) any later version.
#
#   HTERM 3.0 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
#   even the implied warranty ofMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with HTERM 3.0. 
#   If not, see <http://www.gnu.org/licenses/>.
###########################################################################################################
from PyQt4.QtGui import QPushButton
from PyQt4.QtGui import QFileDialog
from PyQt4.QtGui import QMessageBox
from PyQt4.QtGui import QStandardItemModel
from PyQt4.QtGui import QStandardItem
from PyQt4.QtGui import QHeaderView
from PyQt4.QtGui import QAbstractItemView
from PyQt4.QtGui import QTableWidgetItem
from numpy import concatenate
from numpy import nan
from pandas import MultiIndex
from pandas import DataFrame
from pandas import read_pickle  
from pickle import load
from pickle import dump
import ntpath
from os import getcwd

class carga_materiales():
############ FUNCION INICIO ############ 
    def __init__(self, mainWind): # Esta fuencion se ejecuta siempre que se instancia la clase
        #### Defino e inicializo varibles de la clase ####
        self.mainWind=mainWind #esta es la ventana principal, me permite acceder a todo lo visible
        self.lista_de_materiales_propios=''#inicialmente no hay una lista de materiales propios definida
        self.change_BDU(self.lista_de_materiales_propios)#Se indica en el programa el nombre del archivo de base de datos propia que se esta utilizando, solo se puede tener abierta una a la vez. Si esta vacio, sera eso lo que se muestra
        self.lista_de_materiales_original= self.mainWind.resource_path("BDO") #La lista de materiales originales siempre es el archivo llamado BDO que es un data frame, y se encuentra en el directorio raiz de la aplicacion.
        try:        
            self.materiales_originales_dataframe = read_pickle(self.lista_de_materiales_original) #Variable que guarda los materiales "oficiales" en estilo dataframe
        except:
            #No es posible cargar la lista de materiales originales
            msg = QMessageBox() #gerero la ventana
            msg.setWindowTitle('Error')
            msg.setIcon(QMessageBox.Warning) #le pongo el signo de warning (puedo poner alerta y pregunta)
            msg.setText(u'No es posible cargar la lista de materiales.') # mensaje principal del error
            msg.addButton(QPushButton('Aceptar'), QMessageBox.AcceptRole)
            retval = msg.exec_()
            quit()#En este caso el programa se cierra. Esto no deberia pasar nunca ya que se distribuye el ejecutable con la BDO embebida.
        try:
            self.materiales_propios_dataframe = read_pickle(self.lista_de_materiales_propios) #Variable que guarda los materiales propios en estilo dataframe
        except:
            #Si no puedo cargar materiales propios,no doy error solo dejo el data frame vacio para saber que no hay nada cargado
            self.materiales_propios_dataframe=DataFrame()
            # Variables que debo atender en tiempo de programacion
        self.columnas_a_mostrar_en_muro_preview=[2,4,7,5,13,6,14,8,9,15,12,3] #Columnas que se mostraran en el muro
        self.columnas_a_mostrar_en_materiales=[4] #Columnas que se mostraran en materiales. Esto fue hecho a pedido exclusivo del grupo de Arquitectos.
        #### Inicializo componentes ####
        self.refrescar_materiales() #Inicializo el TreeView materiales
        self.inicializo_material_preview() #Inicializo el TableWidget materiales_preview
        #### Conecto componentes con funciones ####
        self.mainWind.materiales.doubleClicked.connect(self.material_dobleclick)#Doble click en un material, esto agrega automaticamente el material al cerramiento (no es necesario apretar agregar capa)
        self.mainWind.materiales.selectionModel().selectionChanged.connect(self.material_click)#con esto se actualiza el pre view solo con seleccionarlo con el teclado raton o lo que sea.
        self.mainWind.guardar_material.clicked.connect(self.guardar_material_clicked)#Se ejecuta cuando se apreta en boton guardar material. Guarda el material que este en pre-view en la BDU (base de datos definida por el usuario)
        self.mainWind.borrar_material.clicked.connect(self.borrar_material_clicked)#Se ejecuta cuando se apreta el boton borrar material. Borra el material de la BDU, si es posible. Si no es posible no hace nada.
        self.mainWind.cargar_materiales.triggered.connect(self.cargar_materiales_triggered)#Se ejecuta cuando se apreta el boton cargar lista de materiales del menu archivo.
        self.mainWind.crear_materiales.triggered.connect(self.crear_materiales_triggered)#Se ejecuta cuando se apreta el boton crear nueva lista de materiales del menu archivo.

############ FUNCIONES INICIALIZACION Y REFRESH ############    

    def change_BDU(self, nombreArchivo):
        #Funcion que modifica el nombre de la BDU (base de datos del usuario, en la interfaz)
        self.mainWind.bdu_label.setText('BDU - {0}'.format(ntpath.basename(nombreArchivo))) #Se modifica el texto
        self.mainWind.bdu_label.setToolTip(nombreArchivo) #Se modifica el tip que aparece al apoyar el cursor sobre el texto. Esto da ademas del nombre del archivo de BDU la ruta de la misma
        
    def refrescar_materiales(self):
        #Funcion que carga en el TreeView lo que este en materiales_originales_dataframe y en materiales_propios_dataframe
        model = QStandardItemModel() #inicializo modelo para el treeview donde se muestran los materiales 
        columnas_para_materiales_seleccionadas_por_usuario = [self.materiales_originales_dataframe.columns.values[i] for i in self.columnas_a_mostrar_en_materiales] #cargo el contenido de las columnas que se van a mostrar en materiales, son seleccionadas por el cliente
        columnas_para_materiales_seleccionadas_por_programador = ['Material'] #Cargo las columnas que no existen pero que las quiero poner igual. En este caso el nombre del material
        columnas_para_materiales_todas = concatenate([columnas_para_materiales_seleccionadas_por_programador, columnas_para_materiales_seleccionadas_por_usuario]) #concateno, y tengo finalmente las columnas que voy a mostrar en materiales
        model.setHorizontalHeaderLabels(columnas_para_materiales_todas) #cargo las columnas a mostrar en el modelo
        self.mainWind.materiales.setModel(model) #cargo el modelo (cargo finalmente las columnas)
        #preparo los datos para cargarlos en la tabla 
        tipos= self.materiales_originales_dataframe.index.get_level_values(0).unique() #En el indice cero estan los tipos de materiales que existen. El unique es para no tener uno par cada material, ya que varios materiales tendran el mismo tipo.
        for tipo in tipos: #Recorro los tipos de materiales
            tipoAgregado = QStandardItem(unicode(tipo))
            model.appendRow(tipoAgregado)#Agrego el tipo de material
            detrodeltipo = self.materiales_originales_dataframe.loc[tipo]#defino un dataframe conteniendo solo las cosas de ese tipo de material            
            subtipos = detrodeltipo.index.get_level_values(0).unique() #Obtengo todos los subtipos que tiene ese tipo. El unique es por lo mismo que antes
            for subtipo in subtipos: #recorro los subtipos
                SubtipoAgregado = QStandardItem(unicode(subtipo))
                tipoAgregado.appendRow (SubtipoAgregado)#Agrego subtipo
                detrodelSubtipo = self.materiales_originales_dataframe.loc[tipo,subtipo]#defino un dataframe que solo tiene los materiales de ese subtipo
                materiales = detrodelSubtipo.index.get_level_values(0).unique()#obtengo la lista de materiales que hay dentro de ese subtipo
                for material in materiales:#recorro los materiales
                    materialAgregado = [QStandardItem(material)]
                    datos_calumnas_a_mostar_en_materiales_seleccionadas_usuario = []                    
                    for i in self.columnas_a_mostrar_en_materiales:
                        aux= self.materiales_originales_dataframe.xs(unicode(material),level=2)[self.materiales_originales_dataframe.columns.values[i]]#obtengo el parametro del material
                        aux= aux.values[0]#Se selecciona la parte del parametro del material correcto.
                        aux=unicode(aux)#se convierte a unicode. Principalmente por compatibilidad multiplataforma
                        if (aux==u'None')|(aux==u'nan'):
                            aux=u''#Si el parametro esta vacio o no es un numero, cargo simplemento el unicode vacio.
                        aux=[QStandardItem(aux)]
                        datos_calumnas_a_mostar_en_materiales_seleccionadas_usuario = concatenate([datos_calumnas_a_mostar_en_materiales_seleccionadas_usuario, aux])#cargo finalmente el parametro
                    datos_calumnas_a_mostar_en_materiales_todas=concatenate([materialAgregado, datos_calumnas_a_mostar_en_materiales_seleccionadas_usuario])
                    SubtipoAgregado.appendRow(datos_calumnas_a_mostar_en_materiales_todas) #cargo el material y sus dos columans de propiedades                    
        #Ahora se cargan los materiales propios del usuario
        if not(self.materiales_propios_dataframe.empty): 
            tipoAgregado = QStandardItem(u'Materiales Cargados')
            model.appendRow(tipoAgregado)
            tipos= self.materiales_propios_dataframe.index.get_level_values(0).unique() #En el indice cero estan los tipos de materiales que existen
            for tipo in tipos: #Recorro los tipos de materiales
                tipoAgregado = QStandardItem(unicode(tipo))
                model.appendRow(tipoAgregado)#Agrego el tipo de material
                detrodeltipo = self.materiales_propios_dataframe.loc[tipo]#defino un dataframe conteniendo solo las cosas de ese tipo de material
                subtipos = detrodeltipo.index.get_level_values(0).unique() #Obtengo todos los subtipos que tiene ese tipo
                for subtipo in subtipos: #recorro los subtipos
                    SubtipoAgregado = QStandardItem(unicode(subtipo))
                    tipoAgregado.appendRow (SubtipoAgregado)#Agrego subtipo
                    detrodelSubtipo = self.materiales_propios_dataframe.loc[tipo,subtipo]#defino un dataframe que solo tiene los materiales de ese subtipo
                    materiales = detrodelSubtipo.index.get_level_values(0).unique()#obtengo la lista de materiales que hay dentro de ese subtipo
                    for material in materiales:#recorro los materiales
                        materialAgregado = [QStandardItem(material)]
                        datos_calumnas_a_mostar_en_materiales_seleccionadas_usuario = []                    
                        for i in self.columnas_a_mostrar_en_materiales:
                            aux= self.materiales_propios_dataframe.xs(unicode(material),level=2)[unicode(self.materiales_propios_dataframe.columns.values[i])]                        
                            aux= aux.values[0]
                            aux=unicode(aux)
                            if (aux==u'None')|(aux==u'nan'):
                                aux=u''
                            aux=[QStandardItem(aux)]
                            datos_calumnas_a_mostar_en_materiales_seleccionadas_usuario = concatenate([datos_calumnas_a_mostar_en_materiales_seleccionadas_usuario, aux])
                        datos_calumnas_a_mostar_en_materiales_todas=concatenate([materialAgregado, datos_calumnas_a_mostar_en_materiales_seleccionadas_usuario])
                        SubtipoAgregado.appendRow(datos_calumnas_a_mostar_en_materiales_todas) #cargo el material y sus dos columans de propiedades                            
        self.mainWind.materiales.show()#muestro el treeview
        #Hago que el ancho de columna se adapte
        self.mainWind.materiales.header().setResizeMode(QHeaderView.ResizeToContents)
        self.mainWind.materiales.header().setStretchLastSection(False)
        #Hago que no se pueda editar los valores
        self.mainWind.materiales.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.mainWind.materiales.header().setStretchLastSection(True) #Para que el ancho se ajuste con el tamaño de la ventana.
        self.mainWind.materiales.selectionModel().selectionChanged.connect(self.material_click)#con esto se actualiza el pre view solo con seleccionarlo con el teclado raton o lo que sea. ES NECESARIO LINKEARLO DE NUEVO DEPUES DE LAS MODIFICACIONES REALIZADAS EN ESTA FUNCION.
        
    def inicializo_material_preview(self):    
        #Funcion que inicializa el TableWidget materiales_preview
        self.mainWind.material_preview.setColumnCount(len(self.columnas_a_mostrar_en_muro_preview)+3) #En esta tabla se mestran todos los parametros seleccionados por el cliente (grupo Arquitectos) y +3 (tipo, subtipo, nombre)
        self.mainWind.material_preview.setRowCount(1) #siempre se muestra solo un material en esta tabla
        horHeaders = [] #variable donde se cargan los headers.
        #Primero se cargan los que se agregaron y no son parametros propiamente del material        
        indice=0        
        horHeaders.append('Tipo')
        dato = QTableWidgetItem(u'')
        self.mainWind.material_preview.setItem(0, indice, dato)
        indice=indice+1
        horHeaders.append('Sub-Tipo')
        dato = QTableWidgetItem(u'')
        self.mainWind.material_preview.setItem(0, indice, dato)
        indice=indice+1
        horHeaders.append('Nombre')
        dato = QTableWidgetItem(u'')
        self.mainWind.material_preview.setItem(0, indice, dato)
        indice=indice+1
        #Se agregan todos los parametros (solo headers)
        for j in self.columnas_a_mostrar_en_muro_preview:
            horHeaders.append(self.materiales_originales_dataframe.columns.values[j])
            dato = QTableWidgetItem(u'') #en el caso de esta tabla tambien inicializo las celdas, para que el usuario tenga la posibilidad de escribir a mano de cero si queire un material nuevo.
            self.mainWind.material_preview.setItem(0, indice, dato)  
            indice=indice+1
        self.mainWind.material_preview.setHorizontalHeaderLabels(horHeaders) #Se insertan los headers que se venian definiendo
        self.mainWind.material_preview.resizeColumnsToContents()#se ajusta el ancho de columna a los headers
        self.mainWind.material_preview.show()#Se muestra/refresca la tabla
        self.mainWind.material_preview.setVerticalHeaderLabels(['-']) #El header de la fila es un guion, ya que no tiene sentido la numeracion.
        #Esto hace que al posar el cursor sobre los header de la tabla, aparecen estos tips al costado el cursor.
        self.mainWind.material_preview.horizontalHeaderItem(3).setToolTip(u"Espesor")
        self.mainWind.material_preview.horizontalHeaderItem(4).setToolTip(u"Densidad")
        self.mainWind.material_preview.horizontalHeaderItem(5).setToolTip(u"Masa")
        self.mainWind.material_preview.horizontalHeaderItem(6).setToolTip(u"Conductividad térmica")
        self.mainWind.material_preview.horizontalHeaderItem(7).setToolTip(u"Calor específico")
        self.mainWind.material_preview.horizontalHeaderItem(8).setToolTip(u"Resistencia térmica")
        self.mainWind.material_preview.horizontalHeaderItem(9).setToolTip(u"Capacidad térmica media")
        self.mainWind.material_preview.horizontalHeaderItem(10).setToolTip(u"Permeabilidad al vapor agua")
        self.mainWind.material_preview.horizontalHeaderItem(11).setToolTip(u"Resistencia al vapor de agua")
        self.mainWind.material_preview.horizontalHeaderItem(12).setToolTip(u"Permeancia al vapor de agua")
        self.mainWind.material_preview.horizontalHeaderItem(13).setToolTip(u"Factor de resistencia al vapor de agua")
        self.mainWind.material_preview.horizontalHeaderItem(14).setToolTip(u"Espesor de aire equivalente SD")

############ FUNCIONES DIRECTAMENTE ASOCIADAS A COMPONENTES ############ 

    def material_dobleclick(self):
        #Funcion que se ejecuta al hacer doble click en un material.
        index = self.mainWind.materiales.selectedIndexes()[0] #Se identifica el indice seleccionado
        material = self.mainWind.materiales.model().itemFromIndex(index).text() #Se obtiene el nombre del material
        if not(self.mainWind.materiales.model().itemFromIndex(index).hasChildren()):#Se verifica que efectivamente se trata de un material y no de un tipo o subtipo
            self.mainWind.generador_muro_obj.agregar_capa_clicked()#Se ejecuta la funcion agregar_capa que es de la clase generador_muro. Notar que esta funcion carga lo que se encuentre en el preview de materiales. Ese preview se carga con el primer click en el material, el segundo hace esto.
            
    def material_click(self):   
        #Funcion asociada a el click en el TreeView materiales. Esta funcion solo carga el material seleccionado en el preview.
        index = self.mainWind.materiales.selectedIndexes()[0] #Se obtiene el indice seleccionado
        material = self.mainWind.materiales.model().itemFromIndex(index).text() #Se obtiene el nombre de la primer columna. 
        indice_tmp=0 #indice utilizado para recorrer las columnas.
        cargo=False #indica si lo seleccionado es o no un material, puede ser un tipo o subtipo. En cuyo caso no se realiza ninguna accion.
        #si lo que toque es un material lo cargo, si no no hago nada.        
        if (not(self.mainWind.materiales.model().itemFromIndex(index).hasChildren()))&(not(self.mainWind.materiales.model().itemFromIndex(index).text()==u'Materiales Cargados')): #Se verifica que no se haya hecho click en un tipo, subtipo, o en el titulo "Materiales Cargados" que indica el comienzo de la BDU
            df=self.materiales_originales_dataframe #Si se selecciono un material efectivamente, cargo en df el dataframe de donde se tienen que sacar los parametros
            cargo=True #Se indica que efectivamente hay algo que cargar
        if not(self.materiales_propios_dataframe.empty):#Se verifica si existe una BDU
            if unicode(material) in self.materiales_propios_dataframe.index.get_level_values(2).unique(): #pregunto si es un material propio efectivamente, en caso de no serlo no hago nada. Ya que puede ser un tipo, subtipo o estar en la BDO
                df=self.materiales_propios_dataframe #Si el material era de la BDU, se sobre escribe df ya q en esta se habia cargado eventualmente la BDO
                cargo=True #Se indica que efectivamente hay algo que cargar
        if cargo==True:
            aux=df.xs(unicode(material),level=2).index.get_level_values(0)[0]#Se obtiene el tipo del material a cargar
            newitem = QTableWidgetItem(unicode(aux))
            self.mainWind.material_preview.setItem (0, indice_tmp, newitem) #primero cargo en la primer columna el tipo del material
            indice_tmp=indice_tmp+1 #aumento el indice que voy a usar para recorrer las columnas
            #despues el sub tipo            
            aux=df.xs(unicode(material),level=2).index.get_level_values(1)[0]
            newitem = QTableWidgetItem(unicode(aux))
            self.mainWind.material_preview.setItem (0, indice_tmp, newitem) #primero cargo en la primer columna el nombre del material
            indice_tmp=indice_tmp+1 #aumento el indice que voy a usar para recorrer las columnas
            #luego el nomrbe del material
            newitem = QTableWidgetItem(unicode(material))
            self.mainWind.material_preview.setItem (0, indice_tmp, newitem) #primero cargo en la primer columna el nombre del material
            indice_tmp=indice_tmp+1 #aumento el indice que voy a usar para recorrer las columnas
            #y ahora el resto de las columnas (parametros del material)
            for j in self.columnas_a_mostrar_en_muro_preview: #notar que j tiene el indice del dataframe no el indice de las columans que se ven, ese es indice_tmp
                contenido = df.xs(unicode(material),level=2)[df.columns.values[j]].values[0] #obtengo la info de la columna que quiero cargar
                if (contenido==None)|(unicode(contenido)==unicode(nan)):
                    contenido=u''#en caso q el parametro no sea correcto, se carga el unicode vacio.
                newitem = QTableWidgetItem(unicode(contenido)) #creo el item nuevo
                self.mainWind.material_preview.setItem (0, indice_tmp, newitem) #se lo incerto en el lugar correspondiente de la tabla
                indice_tmp=indice_tmp+1 #incremento el indice para en la proxima iteracion cargar la siguiente columna
            self.mainWind.material_preview.resizeColumnsToContents() #Ajsuto el ancho de las columnas
            self.mainWind.material_preview.show()
        self.mainWind.materiales.header().setStretchLastSection(True) #Para que el ancho se ajuste con el tamaño de la ventana.

    def guardar_material_clicked(self):   
        #Funcion asociada a el click en el PushBotton guardar_material
        #Preguntar si tengo cargado/creado un archivo de materiales propios. Cargarlo o crearlo si no hay nada (llamo a las funciones cargar o guardar materiales propios)
        if len(self.materiales_propios_dataframe.columns)==0: #esto pasa si estas tratando de guardar un material pero aun no cargaste un archivo de materiales propios
            #pregunto si quieren cargar un archivo de materiales propios existente, crear uno nuevo, o cancelar
            msg = QMessageBox() #gerero la ventana
            msg.setIcon(QMessageBox.Warning) #le pongo el signo de warning (puedo poner alerta y pregunta)
            msg.setText(u'No cargó ningún archivo de materiales.') # mensaje principal del error
            msg.setWindowTitle('Error')
            botoncargar=msg.addButton(QMessageBox.Yes)
            botoncargar.setText('Cargar')
            botonnuevo=msg.addButton(QMessageBox.No)
            botonnuevo.setText('Crear nuevo')
            botoncancelar=msg.addButton(QMessageBox.Cancel)
            botoncancelar.setText('Cancelar')
            retval = msg.exec_() # Ejecuto El mensaje
            if msg.clickedButton()==botonnuevo:#Si el usuario quiere crear una BDU nueva
                self.crear_materiales_triggered() #Se ejecuta misma funcion que al hacer click en crear nueva lista de materiales del menu archivo
            if msg.clickedButton()==botoncargar:#Si el usuario quiere cargar una BDU existente
                self.cargar_materiales_triggered()#Se ejecuta la misma funcion que al hacer click en cargar lista de materiales
        if not(len(self.materiales_propios_dataframe.columns)==0):#Si efectivamente hay ya un BDU cargada
            material_a_agregar = [self.mainWind.material_preview.item(0,i+2).text() for i in range(len(self.columnas_a_mostrar_en_muro_preview)+1)]#se obtiene el nombre y todos los parametros de dicho material   
            [estaono, esigual]=self.chequeo_si_material_existe(self.materiales_originales_dataframe,material_a_agregar)#Se identifica si el material pertenece a la base de datos original BDO
            if material_a_agregar[0]==u'Materiales Cargados':#si intento generar un material con este nombre
                estaono=True #Impongo como que si el material existiera en BDO ya que ese nombre esta reservado para el titulo de la BDU.
            if estaono==True:#estaono indica si el nombre de material ya esta utilizado en BDO, si el nombre ya esta utilizado:
                msg = QMessageBox() #gerero la ventana
                msg.setWindowTitle('Error')
                msg.setIcon(QMessageBox.Warning) #signo de warning
                msg.setText(u'El nombre del material está reservado.') # mensaje principal del error
                msg.addButton(QPushButton('Aceptar'), QMessageBox.AcceptRole)
                retval = msg.exec_() # Ejecuto el mensaje
            else:#Si el nombre no esta reservado por BDO
                [estaono, esigual]=self.chequeo_si_material_existe(self.materiales_propios_dataframe,material_a_agregar)#verifico si el nombre no esta ya en uso en BDU
                if estaono==True:#estaono indica si el nombre esta o no en uso en BDU
                    #Le pregunto al usuario si lo quiere borrar o no
                    msg = QMessageBox() #gerero la ventana
                    msg.setIcon(QMessageBox.Warning) #Signo de warning
                    msg.setWindowTitle('Error')
                    msg.setText(u'Ya existe un material con ese nombre en su base de datos, ¿desea eliminarlo?') # mensaje principal del error
                    botonsi=msg.addButton(QMessageBox.Yes)
                    botonsi.setText('Si')
                    botonno=msg.addButton(QMessageBox.No)
                    botonno.setText('No')
                    retval = msg.exec_() # Ejecuto mensaje
                    if msg.clickedButton()==botonsi:#Si el usuario decide borrarlo
                        queresborrarlo=True#activo bandera borrar
                    else:
                        queresborrarlo=False#Si no quiere borrarlo desactivo badera borrar
                    if queresborrarlo==True:#Si el usuario decidio borrar el material con nombre conflictivo
                        self.materiales_propios_dataframe=self.materiales_propios_dataframe.drop(unicode(material_a_agregar[0]),level=2)#simplemente se elimina dicho material del dataframe
                        estaono=False #Ahora no esta mas
                if estaono==False:#Esto puede ser porque nunca estuvo, o porque estuvo y el usuario decidio borrarlo    
                    #generlo lo que necesito para formar el nuevo material
                    columnas=self.materiales_originales_dataframe.columns #las columnas, voy a mantener las mismas que los originales
                    cantidad_columnas=len(self.materiales_originales_dataframe.columns)                
                    indices=[[unicode(self.mainWind.material_preview.item(0,i).text())] for i in range(3)] #los indices, son los primeros tres, en ese orden del preview
                    datos=[[None]*cantidad_columnas] #creo los datos por ahora vacios
                    for i in range(len(self.columnas_a_mostrar_en_muro_preview)):
                        datos[0][self.columnas_a_mostrar_en_muro_preview[i]]=unicode(material_a_agregar[i+1])#cargo los datos en la variable auxiliar datos
                    df = DataFrame(datos, index=MultiIndex.from_tuples(zip(*indices)), columns=columnas) #genero el nuevo elemento, genero el dataframe
                    self.materiales_propios_dataframe=self.materiales_propios_dataframe.append(df) #agrego el nuevo elemento a mis materiales (BDU)
                    archivo = open(self.lista_de_materiales_propios, 'w') #Creo que archivo en el lugar y con el nombre que la persona quiere
                    dump(self.materiales_propios_dataframe, archivo) #uso la herramienta pickle para guardar la variable materiales_propios_dataframe
                    archivo.close() #cierro el archivo
                    #llama a la funcion que refresca el treeview
                    self.refrescar_materiales()
              
    def borrar_material_clicked(self):   
        if not(self.mainWind.materiales.selectedIndexes()==[]): #pregunto si hay algo seleccionado
            if not(self.materiales_propios_dataframe.empty): #me aseguro que tenga materiales propios cargados antes de preuguntar por ellos
                indice = self.mainWind.materiales.selectedIndexes()[0] #Se obtiene indice de material seleccionado
                material = self.mainWind.materiales.model().itemFromIndex(indice).text() #obtengo el nombre de lo que esta seleccionado
                if not(self.mainWind.materiales.model().itemFromIndex(indice).hasChildren()):#Me aseguro que es un material y no un tipo, o subtipo
                    self.materiales_propios_dataframe=self.materiales_propios_dataframe.drop(unicode(material), level=2) #elimino ese material de materiales propios
                    archivo = open(self.lista_de_materiales_propios, 'w') #abro la lista de materiales propios BDU
                    dump(self.materiales_propios_dataframe, archivo) #uso la herramienta pickle para guardar la variable materiales_propios_dataframe
                    archivo.close() #cierro el archivo
                    #llama a la funcion que refresca el treeview
                    self.refrescar_materiales() 
        
    def cargar_materiales_triggered(self):   
        #Funcion asociada a el trigger en el PushBotton MenuBar button cargar_materiales
        cwd=getcwd() #obtengo el current working directory
        dialogo = QFileDialog() #creo que dialogo 
        dialogo.setWindowTitle('Cargar lista de materiales') #titulo 
        dialogo.setDirectory(cwd) #se abre en el directorio donde esta el ejecutable
        dialogo.setAcceptMode(QFileDialog.AcceptOpen)
        dialogo.setNameFilter('Materiales (*.materiales)') #solo muestro los archivos con la extension .materiales
        dialogo.setDefaultSuffix('materiales') #si el usuario no lo hace, se le agrega igual la extension .materiales
        if dialogo.exec_() == QFileDialog.Accepted:
            fname=dialogo.selectedFiles()[0] #si se competa la accion se guarda en fname la ruta al archivo cargado
        else:
            fname=u'' #si se cancela o cierra la accion cargo el unicode vacio en fname
        if not fname==u'': #si la persona cancela, el nombre del archivo va a ser vacio.
            try:            
                archivo = open(fname, 'r') #abro el archivo que la persona selecciono
                self.materiales_propios_dataframe=load(archivo) #uso pickle para cargar el archivo ya que el mismo fue generado con la misma herramienta 
                archivo.close() #cierro el archivo
                self.lista_de_materiales_propios=fname #guardo la ruta del archivo de BDU
                self.change_BDU(self.lista_de_materiales_propios) #Muestro el numbre y ruta de la BDU en la interfaz
            except:
                pass #Si esto da un error, por ejemplo porque el archivo que se esta queriendo cargar esta dañado, no hago nada. Notar que actualmente no se da un error.
            self.refrescar_materiales() #se refresca la lista de materiales
    def crear_materiales_triggered(self):           
        #Si quieren uno nuevo primero deben dar el nombre
        cwd=getcwd() #obtengo el current working directory
        dialogo = QFileDialog() #creo que dialogo 
        dialogo.setWindowTitle('Crear lista de materiales') #titulo
        dialogo.setDirectory(cwd) #se abre en el directorio donde esta el ejecutable
        dialogo.setAcceptMode(QFileDialog.AcceptSave)
        dialogo.setNameFilter('Materiales (*.materiales)')#solo muestro los archivos con la extension .materiales
        dialogo.setDefaultSuffix('materiales')#si el usuario no lo hace, se le agrega igual la extension .materiales
        if dialogo.exec_() == QFileDialog.Accepted:
            fname=dialogo.selectedFiles()[0] #si se competa la accion se guarda en fname la ruta al archivo cargado
        else:
            fname=u''   #si se cancela o cierra la accion cargo el unicode vacio en fname     
        if not fname==u'': #si la persona cancela el nombre va a estar vacio                    
            self.lista_de_materiales_propios='%s' % fname #Guardo el nombre del archivo que estoy usando
            self.change_BDU(self.lista_de_materiales_propios)
            archivo = open(self.lista_de_materiales_propios, 'w') #Creo que archivo en el lugar y con el nombre que la persona quiere, y le agrego la extension .materiales
            self.materiales_propios_dataframe=DataFrame([],[[],[],[]],self.materiales_originales_dataframe.columns)#al cargarle esto, indico que tengo un archivo donde guardar, activo. no es lo mismo estar asi que estar con el dataframe empty. si esta empty indica que no hay archivo
            dump(self.materiales_propios_dataframe, archivo) #uso la herramienta pickle para guardar
            archivo.close() #cierro el archivo 
            self.refrescar_materiales() #se refresca la lista de materiales
############ FUNCIONES AUXILIARES ############ 
    
    def chequeo_si_material_existe(self, df_tocheck, material):
        #Funcion que chequea si un material esta en el dataframe df_tocheck. Se utiliza por varias funciones
        #Asumo que material tiene la misma cantidad de columnas y en orden de self.columnas_a_mostrar_en_muro_preview +1 ya que al arranque tiene el nombre
        #devuelo mas de una cosa:
        # [Booleano si esta o no el nombre, Boleano si es igual o no]
        if not(df_tocheck.empty): #Primero se asegura que el dataframe no este vacio.
            estaono = unicode(material[0]) in df_tocheck.index.get_level_values(2).unique() #True su el nombre de l material esta entre los nombres de matriales que hay en el dataframe
            if estaono==True:#Si el material esta, verifico si material es igual al material con ese nombre contenido en el databrame (df_tochek)
                esigual=True#Es igual hasta que se demuestre lo contrario
                for i in range(len(material)-1): #para cada columna del material
                    aux1=material[i+1] #cargo el valor de la columna del material
                    aux2=df_tocheck.xs(unicode(material[0]),level=2)[df_tocheck.columns.values[self.columnas_a_mostrar_en_muro_preview[i]]].values[0]#obtengo el parametro del material del dataframe
                    aux1=self.mainWind.generador_muro_obj.formato_numeros(aux1)#formateo el valor de la columna del material
                    aux2=self.mainWind.generador_muro_obj.formato_numeros(aux2)#aplico el mismo formato a la columna del material en el dataframe
                    if (aux2==unicode(nan))|((unicode(aux2)==u'None')|(unicode(aux2)==u'nan')|(unicode(aux2)==u'')|(unicode(aux2)==u'-')):
                        aux2=aux1 #si el material del dataframe no tiene un parametro valido para esa columna (i), entonces considero que son iguales a efectos de la comparacion. Notar que el dataframe no tiene porque tener autocompletados todos sus parametros mientras que el material si puede llegar a tenerlo.
                    if not(unicode(aux1)==unicode(aux2)):
                        esigual=False#si alguna de las columnas no coincide, todo el material se considera diferente al guardado en el dataframe.
            else:
                esigual=False #Si el material no esta, considero que no es igual
        else:
            esigual=False #si el dataframe es vacio, entonces el material no es igual
            estaono=False #ni esta en el dataframe
        return [estaono, esigual] #retorno parametros
