# -*- coding: utf-8 -*-
###########################################################################################################
#   Universidad de la República, Facultad de Ingeniería, Instituto de Ingeniería Eléctrica 
#
#   Copyright 2017 Universidad de la República, Facultad de Ingeniería, Instituto de Ingeniería Eléctrica 
#
#   Autores: Pablo Pérez-Nicoli, Francisco Veirano, Juan Pablo Oliver
#
#   Version: 1.0
#
#   This file is part of HTERM 3.0.
#
#   HTERM 3.0 is free software: you can redistribute it and/or modify it under the terms of the GNU 
#	 General Public License as published by the Free Software Foundation, either #version 3 of the 
#   License, or (at your option) any later version.
#
#   HTERM 3.0 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
#   even the implied warranty ofMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with HTERM 3.0. 
#   If not, see <http://www.gnu.org/licenses/>.
###########################################################################################################
from PyQt4 import QtCore
from PyQt4.QtGui import QIcon
from PyQt4.QtGui import QApplication
from sys import exit, argv
import sys
from PyQt4.uic import loadUiType

import listas_materiales as listas_materiales
import generador_muro as generador_muro
import muestroDatos as muesDatos
import Acerca_de as Acerca_de
import core as core
import imprimir as imp
from os import path as ph
import conversor as conv
import starting as st

from PyQt4.QtGui import QMessageBox
from PyQt4.QtGui import QPushButton
from locale import getdefaultlocale

#Clase para poder acceder a archivos incluso depues de haber compilado y embebido los archivos
def resource_path(relative_path):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
    except Exception:
        base_path = ph.abspath(".")

    return ph.join(base_path, relative_path)
    
#Se carga inferfaz grafica
securitySearchForm = resource_path("window.ui")
Ui_MainWindow, QMainWindow = loadUiType(securitySearchForm)

#Clase Main
class Main(QMainWindow, Ui_MainWindow):
    def __init__(self, ):
        
        super(Main, self).__init__()
        self.setupUi(self)
        #Clases que componen el software:
        self.lista_materiales_obj = listas_materiales.carga_materiales(self) #Clase que controla las listas de materiales, es responsable por mostrarla, y pre-cargarla en el material preview. Tambien permite crear y administrar listas de materiales porpias.
        self.generador_muro_obj = generador_muro.generador_muro(self) #Clase que permite la creacion del muro, esto incluye agregar, borrar, sustituir capaz. Tambien es responsable por verificar el contenido del muro, y autocompletar parametros redundantes. Esta clase, tambien permite guardar y gargar muros creados. 
        self.calcObj = core.Calculos(self) #Clase encargada de realizar los calculos
        self.muestroDatosObj = muesDatos.muestroDatos(self) #Clase encargada de presentar los resultados en las graficas
        self.imprimirObj = imp.generadorPDF(self) #Clase encargada de exportar los datos a formato .pdf
        self.acercade = Acerca_de.acercade(self) #Clase encargada de manejar la ayuda, acerca de, y mostrar otra informacion relevante para asistir a el usuario
        self.conversor = conv.conversor(self) #Clase que permite realizar coversion de unidades, en esta version del software solo se incluye conversor para la permeabilidad de vapor de agua
        
        self.nombreArchivo = u'Sin Nombre *' #Titulo inicial de la ventana, hasta que no se guarde un muro. El asterisco al final es importante porque es lo que indica que el cerramietno acual aun no fue guardado.
        self.changeTitle(self.nombreArchivo) 
        
    #Clase para poder acceder a archivos incluso depues de haber compilado y embebido los archivos
    #Se define como funcion de clase main para que sea accesible por todas las clases. Es necesario que esta funcion se encuentre definida en el main.
    def resource_path(self, relative_path):
            """ Get absolute path to resource, works for dev and for PyInstaller """
            try:
                # PyInstaller creates a temp folder and stores path in _MEIPASS
                base_path = sys._MEIPASS
            except Exception:
                base_path = ph.abspath(".")
        
            return ph.join(base_path, relative_path)
    
    #Funcion que se ejecutara al intentar cerrar el programa. Permite preguntar a usuario si desea guardar el muro.
    def closeEvent(self, event):
        nombre = self.windowTitle() #se obtiene el titulo actual de la ventana
        aster = nombre[-1] #Se obtiene el ultimo caracter del titulo
        if aster=="*": #Si el ultimo caracter es un asterisco, en tal caso hay que preguntar al usuario si desea guardar. Si el nombre no termina en asterisco, quiere decir que el cerramiento ya fue guardado (en ese caso no se hace nada)
            msg = QMessageBox() #genero la ventana
            msg.setIcon(QMessageBox.Question) #le pongo el signo de warning
            yesbut = QPushButton('Si') #se le define boton de si
            nobut = QPushButton('No') #se le define boton de no
            canbut = QPushButton('Cancelar') #se le define boton de Cancelar
            msg.addButton(yesbut, msg.YesRole) 
            msg.addButton(nobut, msg.NoRole)
            msg.addButton(canbut, msg.RejectRole)
            quit_msg = u'¿Desea guardar el cerramiento antes de cerrar HTERM?' #texto del mensaje
            msg.setText(quit_msg)
            msg.setWindowTitle(u'Mensaje') #Titulo de la ventana
            msg.exec_()
            
            if msg.clickedButton()  == yesbut:
                self.generador_muro_obj.guardar_muro_triggered()#Si el usuario desea guardar, se llama a la misma funcion que se ejecuta al hacer click en Guardar Cerramiento en menu archivo
                event.accept()
            elif msg.clickedButton()  == nobut:
                event.accept() #Si dice no, no se hace nada y se cierra el programa
            else:
                event.ignore() #Si dice cancelar o cierra, no se hace nada ni se cierra el programa
        else:
            event.accept() 
    
    def changeTitle(self, nombreArchivo):
        self.setWindowTitle('HTERM 3.0 - {0}'.format(nombreArchivo))
    
if __name__ == '__main__':
    inicio = st.starting(resource_path)#Se muestra la imagen de inicio con creditos
    app = QApplication(argv) #Se inicia la aplicacion
    
    securitySearchForm = resource_path(u'qt_es.qm')#Se cargan textos en espanol
    translator_qt = QtCore.QTranslator() #Se traducen las interfaces
    translator_qt.load(securitySearchForm) #Se traducen las interfaces
    app.installTranslator(translator_qt) #Se traducen las interfaces 
    
    main = Main() #Se inicia el Main
    main.showMaximized() #inicia maximizado
    securitySearchForm = resource_path(u'icono-hterm3.ico') #Se carga el icono que se muestra
    app.setWindowIcon(QIcon(securitySearchForm)) #Se inserta el icono de la aplicacion
    main.setWindowIcon(QIcon(securitySearchForm)) #Se inserta el icono de la ventana  
    
    exit(app.exec_()) #esto se comenta para tener el prompt cuando se ejecuta en terminal
