# -*- coding: utf-8 -*-
###########################################################################################################
#   Universidad de la República, Facultad de Ingeniería, Instituto de Ingeniería Eléctrica 
#
#   Copyright 2017 Universidad de la República, Facultad de Ingeniería, Instituto de Ingeniería Eléctrica 
#
#   Autores: Pablo Pérez-Nicoli, Francisco Veirano, Juan Pablo Oliver
#
#   Version: 1.0
#
#   This file is part of HTERM 3.0.
#
#   HTERM 3.0 is free software: you can redistribute it and/or modify it under the terms of the GNU 
#	 General Public License as published by the Free Software Foundation, either #version 3 of the 
#   License, or (at your option) any later version.
#
#   HTERM 3.0 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
#   even the implied warranty ofMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with HTERM 3.0. 
#   If not, see <http://www.gnu.org/licenses/>.
###########################################################################################################
from matplotlib import rc
from matplotlib.figure import Figure
from matplotlib.backends.backend_qt4agg import (
    FigureCanvasQTAgg as FigureCanvas)
from numpy import array
import obtengoParametros as obtParam
from copy import deepcopy
from PyQt4.QtGui import QSizePolicy
from PyQt4.QtGui import QTableWidgetItem
from PyQt4.QtGui import QMessageBox
from PyQt4.QtGui import QPushButton

class muestroDatos():
    def __init__(self,mainWind):

        # Inicializo la figura de condensacion
        self.fig = Figure( )       
        self.ax1f1 = self.fig.add_subplot(111)
        self.ax1f1.axis('off')
        rect = self.fig.patch
        rect.set_facecolor('white')
        mainWind.graf = FigureCanvas(self.fig)
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        mainWind.graf.setSizePolicy(sizePolicy)                
        mainWind.pltGraf.addWidget(mainWind.graf)
        mainWind.centralwidget.setSizePolicy(sizePolicy)
        mainWind.setSizePolicy(sizePolicy)
        mainWind.graf.draw()
        mainWind.graf.show()
        # Inicializo figura de amortiguamiento
        self.figamor = Figure()       
        self.axfigamor = self.figamor.add_subplot(111)      
        self.axfigamor.axis('off')
        rectfigamor = self.figamor.patch
        rectfigamor.set_facecolor('white')                 
        mainWind.grafamor = FigureCanvas(self.figamor)
        mainWind.grafamor.setSizePolicy(sizePolicy)                
        mainWind.pltAmor.addWidget(mainWind.grafamor)        
        mainWind.grafamor.draw()
        mainWind.grafamor.show()
        # inicializacion de la tabla
        mainWind.tabla.setColumnCount(3)         
        mainWind.tabla.setHorizontalHeaderLabels([u'Plano',u't [°C]',u'tr [°C]']) 
        # objecto con funciones para manejo de parametros
        self.paramObj = obtParam.manejoParametros(mainWind)      
        # se conecta el boton de calcular con la función que actualiza todos los datos
        mainWind.plotButton.clicked.connect(self.actualizoDatos)

        self.mainWind=mainWind

        
    def clearFig(self):
    # Función que borra todos los datos presentes en la ventana
        self.fig.clf()
        self.ax1f1.axis('off')
        self.figamor.clf()
        self.axfigamor.axis('off')
        self.mainWind.graf.draw()
        self.mainWind.grafamor.draw()
        self.mainWind.tabla.clearContents()
        self.mainWind.masa.setText(u'Masa')          
        self.mainWind.trans.setText(u'Transmitancia')
        self.mainWind.espesor.setText(u'Espesor')   
        self.mainWind.amortiguamiento.setText(u'Factor de Amortiguación')   
        self.mainWind.fase.setText(u'Retardo Térmico')  
        
    def graficarMuro(self,temp,tempVap,espesores):
        # Seteo aspectos de apariencia de la gráfica
        font = {'size'   : 11}
        rc('font', **font)
        letra = 11
        self.fig.clf()
        self.ax1f1 = self.fig.add_subplot(111)                   
        
        # Ancho total del muro
        anchomuro=sum(espesores)
        
        # Primero genero el vector de x de la figura. estos son cada uno de los espesores pero también las alitas del muro
        # ver figura
        # Agrego el alita del muro
        vec_espe = [0]                      # aca va la temp interior
        vec_espe.append(0.2*anchomuro)      # aca a va otra tem interior
        vec_espe.append(0.3*anchomuro)      # aca la temp entre capa int y muro 
        # Genero vector de espesores para temperaturas
        for i in range(len(espesores)): vec_espe.append(vec_espe[-1]+espesores[i])
        vec_espe.append(vec_espe[-1]+0.1*anchomuro) # aca la temp exterior 
        vec_espe.append(vec_espe[-1]+0.2*anchomuro) # aca la segunda temp exterior           
        
        # Genero vector de espesores de muro con alita este se usa para graficar las horizontales de arriba y abajo
        vec_espe2 = vec_espe[:]
        del vec_espe2[0]
        del vec_espe2[-1]
        # Genero vector de espesores de muro sin alita este se usa para las temperaturas de rocio y para las rayas verticales 
        # que indican donde estan las capas
        vec_espe3 = vec_espe2[:]
        del vec_espe3[0]
        del vec_espe3[-1]
        
        # parametrizo un poco en funcion del as temperaturas que tengo que graficar
        rangosup=max([max(temp) + (max(temp) - min(temp))*0.2,max(tempVap) + (max(tempVap) - min(tempVap))*0.2])
        rangoinf= min([min(temp)-(max(temp) - min(temp))*0.2,min(tempVap)-(max(tempVap) - min(tempVap))*0.2])
        
        # Vector de temperaturas con primer y ultimo elemento repetido
        vec_temp = temp
        vec_temp.insert(0,temp[0])
        vec_temp.insert(-1,temp[-1])
        
        # graficamos als paredes
        for i in range(len(vec_espe3)): self.ax1f1.plot([vec_espe3[i],vec_espe3[i]],[rangoinf,rangosup],'k',linewidth=3.0)
        
        #graficamos las temperaturas 
        templeg, = self.ax1f1.plot(vec_espe,vec_temp,color='#1E2C89',linewidth=3.0)
        tempr_leg, = self.ax1f1.plot(vec_espe3,tempVap,color='#19805E',linestyle='--',linewidth=3.0)
    
        #graficamos el techo y piso
        self.ax1f1.plot([vec_espe2[0],vec_espe2[-1]],[rangosup,rangosup],'k',linewidth=3.0)
        self.ax1f1.plot([vec_espe2[0],vec_espe2[-1]],[rangoinf,rangoinf],'k',linewidth=3.0)
        self.ax1f1.set_xlim([vec_espe[0],vec_espe[-1]])
        self.ax1f1.set_ylim([rangoinf,rangosup])
        
    
        # escribimos las temperaturas externas e internas
        self.ax1f1.text(vec_espe[0], vec_temp[0]+(rangosup-rangoinf)*0.05,u'ti={} °C'.format(vec_temp[0]), wrap=True, horizontalalignment='center', fontsize=letra)
        self.ax1f1.text(vec_espe[-1],  vec_temp[-1]+(rangosup-rangoinf)*0.05,u'te={} °C'.format(vec_temp[-1]), wrap=True, horizontalalignment='center', fontsize=letra)
              
        # escribimos los numeros de cpas abajo de cada capa
        for i in range(len(espesores)):
            self.ax1f1.text((vec_espe[i+2]+vec_espe[i+3])/2, rangoinf-abs((rangosup-rangoinf))*0.05,u'{}'.format(i+1), wrap=True, horizontalalignment='center', fontsize=letra)
            
        # Nos fijamos si el muro tiene camaras de aire muy ventiladas. Si tiene nos quedamos solo
        # el indice de la primera que aprezca empezando desde el interior. 
        camaraAire= self.mainWind.generador_muro_obj.camara_MV # Indices donde se encuentran cada capa que es camara de aire muy ventilada  
        # Para que sea más facil paso la lista a un numero y none si esta vacio.
        if camaraAire==[]: camaraAire=None 
        else: camaraAire=camaraAire[0] 
        
        # Escribimos exterior e interior o camara de aire dependiendo del caso
        if temp[0]>=temp[-1]:
            self.ax1f1.text(vec_espe[0], vec_temp[0]-(rangosup-rangoinf)*0.1,"Interior", wrap=True, horizontalalignment='center', fontsize=letra)
            if camaraAire==None :
                self.ax1f1.text(vec_espe[-1],  vec_temp[-1]-(rangosup-rangoinf)*0.1,"Exterior", wrap=True, horizontalalignment='center', fontsize=letra)        
            else:
                self.ax1f1.text(vec_espe[-1],  vec_temp[-1]-(rangosup-rangoinf)*0.1,u'Cámara de Aire', wrap=True, horizontalalignment='center', fontsize=letra)        
            
        else:
            self.ax1f1.text(vec_espe[0], vec_temp[0]-(rangosup-rangoinf)*0.1,"Interior", wrap=True, horizontalalignment='center', fontsize=letra)
            if camaraAire==None :
                self.ax1f1.text(vec_espe[-1],  vec_temp[-1]-(rangosup-rangoinf)*0.1,"Exterior", wrap=True, horizontalalignment='center', fontsize=letra)        
            else:
                self.ax1f1.text(vec_espe[-1],  vec_temp[-1]-(rangosup-rangoinf)*0.1,u'Cámara de Aire', wrap=True, horizontalalignment='center', fontsize=letra)        
        
        
        # Ahora hay que pintar de gris si hay condensacion
        vec_temp_aux = vec_temp[:]
        del vec_temp_aux[0]
        del vec_temp_aux[-1]        
        del vec_temp_aux[0]
        del vec_temp_aux[-1]      
            
        vec_temp_aux_arr = array(vec_temp_aux)
        vec_tempsvap_arr = array(tempVap)
        
        fill_leg =self.ax1f1.fill_between(vec_espe3,vec_temp_aux_arr,vec_tempsvap_arr,where=vec_temp_aux_arr<=vec_tempsvap_arr,facecolor='grey', interpolate=True)        
        
        # ponemos el legend
        self.ax1f1.legend([templeg,tempr_leg,fill_leg],["t","tr",u'Condensación'],loc='upper center',bbox_to_anchor=(0.5,1.13), ncol=3, fancybox=True, shadow=True)

        # apagamos los ejes y ajustamos el tamaño
        self.ax1f1.axis('off')
        self.fig.tight_layout(rect=(0.01,-0.01,0.95,0.93))
    
    def cargoTabla(self, temp,tempsvap,mainWind): 
    # Funcion que carga datos en la tabla    
        mainWind.tabla.clearContents()
        # Cargo Tabla        
        tempaux = temp[:]
        del tempaux[0]
        del tempaux[-1] 
        del tempaux[0]
        del tempaux[-1]  
        mainWind.tabla.setRowCount(len(tempaux))  
        
        # Pongo el label al lado de cada renglon, incluye diferenciación si es ca o exterior
        label = ['In-1']    
        if len(tempaux) > 2:
            for i in range(len(tempaux)-2):
                label.append('{}-{}'.format(i+1,i+2))
        
        # Nos fijamos si el muro tiene camaras de aire muy ventiladas. Si tiene nos quedamos solo
        # el indice de la primera que aprezca empezando desde el interior. 
        camaraAire= self.mainWind.generador_muro_obj.camara_MV # Indices donde se encuentran cada capa que es camara de aire muy ventilada  
        # Para que sea más facil paso la lista a un numero y none si esta vacio.
        if camaraAire==[]: camaraAire=None 
        else: camaraAire=camaraAire[0] 
        
        if camaraAire==None :
            label.append('{}-Ex'.format(len(tempaux)-1))            
        else:
            label.append('{}-CA'.format(len(tempaux)-1)) 
        
        # cargo cada valor       
        for i in range(len(tempaux)):
            mainWind.tabla.setItem(i, 0, QTableWidgetItem(unicode(label[i])))  
            mainWind.tabla.setItem(i, 1, QTableWidgetItem(unicode(round(tempaux[i],2))))    
            mainWind.tabla.setItem(i, 2, QTableWidgetItem(unicode(round(tempsvap[i],2))))   
        mainWind.tabla.setVerticalHeaderLabels(label) 
        
    def cambioMU(self,espe,M,U,mainWind):   
    # Función que muestra los valores de masa y transmitancia y espesor en los text box correspondientes    
       mainWind.masa.setText(u'Masa: {} Kg/m²'.format(unicode(round(M,2))))          
       mainWind.trans.setText(u'Transmitancia Térmica: {} W/m²K'.format(unicode(round(U,2))))
       mainWind.espesor.setText(u'Espesor: {} m'.format(unicode(round(espe,3))))   
       
    def cambioAmo(self,modamormost,fasemostrar,mainWind):    
    # cambia text box de amortiguamiento y retardo térmico    
       mainWind.amortiguamiento.setText(u'Factor de Amortiguación: {} '.format(unicode(round(modamormost,3))))   
       mainWind.fase.setText(u'Retardo Térmico: {} Hs'.format(unicode(round(fasemostrar,2))))          
       
    def graficoAmor(self,tiempo,tempext,tempint):
        
       self.figamor.clf()          
       self.axfigamor = self.figamor.add_subplot(111)                    
       self.axfigamor.axis('on')
       #tiempo2 = [x/(60*60) for x in tiempo]
       
       tempextleg, = self.axfigamor.plot(tiempo,tempext,color='#19805E',linewidth=3.0)
       tempintleg, =self.axfigamor.plot(tiempo,tempint,color='#1E2C89',linestyle='--',linewidth=3.0)            
       
       self.axfigamor.plot(tiempo,[0] * len(tempint),color='#000000',linestyle='-',linewidth=1.0)              
       
       self.axfigamor.legend([tempextleg,tempintleg],["T Exterior","T Interior"],loc='upper center', bbox_to_anchor=(0.5,1.14), ncol=3, fancybox=True, shadow=True)
       self.axfigamor.set_xlim([tiempo[0],tiempo[-1]])
       self.axfigamor.set_ylabel(u"Variación Temperatura [°C]")
       self.axfigamor.set_xlabel('Tiempo [Hs]')
       self.figamor.tight_layout(rect=(0.01,-0.01,0.95,0.93))
# Evento del boton plot
       
    def pasarMuroFloat(self,muro):
         muroaux = 0   
         muroaux= deepcopy(muro)     
         for i in range(len(muroaux)): 
             for l in range(1, 12): 
                 try: 
                     muroaux[i][l] = float(muroaux[i][l]) 
                 except :
                     pass
         for i in range(len(muroaux)): muroaux[i][1]=muroaux[i][1]/1000       
         return muroaux
       
       
    def actualizoDatos(self):
        
     self.mainWind.generador_muro_obj.muro_modificado(u'nada') #es necesario actualizar el muro antes, porque si movieron una fila, no queda bien actualizado   
     flag = self.mainWind.generador_muro_obj.muro_valido
     capas_malas= self.mainWind.generador_muro_obj.capas_malas
        
     if (flag==True):
         muro = self.pasarMuroFloat(self.mainWind.generador_muro_obj.datos_muro_lista)
         verifcam=self.mainWind.calcObj.verifCamaraAire(muro)
         if (verifcam==True):                            
             self.paramObj.leoParametros(self.mainWind)
             (temp,tempVap,espesores,M,U,espetot) = self.mainWind.calcObj.calcCond(muro)
             (tiempo,tempexterior,tempinterior,nograficoamor,modamormost,fasemostrar) = self.mainWind.calcObj.calcAmor(muro)             
             if nograficoamor==False:
                 self.graficoAmor(tiempo,tempexterior,tempinterior)
                 self.cambioAmo(modamormost,fasemostrar,self.mainWind)   
             else:
                 self.figamor.clf()
                 self.mainWind.amortiguamiento.setText(u'Factor Amortiguamiento')   
                 self.mainWind.fase.setText(u'Retardo')               
             self.graficarMuro(temp,tempVap,espesores)  
             self.mainWind.graf.draw()
             self.mainWind.grafamor.draw()        
             self.cargoTabla(temp,tempVap,self.mainWind)
             self.cambioMU(espetot,M,U,self.mainWind)
         else:
             msg = QMessageBox() #genero la ventana
             msg.setIcon(QMessageBox.Warning) #le pongo el signo de warning (puedo poner alerta y pregunta)
             msg.setWindowTitle(u'Error')
             msg.addButton(QPushButton('Aceptar'), QMessageBox.AcceptRole)
             #msg.setText(u'Las capas {} del cerramiento son incorrectas.'.format(capas_malas)) # mensaje principal del error
             msg.setText(u'Una cámara de aire no puede ser ni la primera ni la última capa del cerramiento.') # mensaje principal del error    
            
             msg.exec_() # No se bien que hace pero es necesario   
     else:
         msg = QMessageBox() #genero la ventana
         msg.setIcon(QMessageBox.Warning) #le pongo el signo de warning (puedo poner alerta y pregunta)
         msg.setWindowTitle(u'Error')
         msg.addButton(QPushButton('Aceptar'), QMessageBox.AcceptRole)
         for i in range(len(capas_malas)): capas_malas[i] = capas_malas[i]+1
         if  len(capas_malas)==0:
               msg.setText(u'El cerramiento se encuentra vacío.'.format(capas_malas)) # mensaje principal del error
         elif len(capas_malas)==1:
               msg.setText(u'La capa {} del cerramiento está incompleta.'.format(capas_malas)) # mensaje principal del error
         else:
               msg.setText(u'Las capas {} del cerramiento están incompletas.'.format(capas_malas)) # mensaje principal del error
             #msg.setDetailedText(u'{} Solo adminte una entrada numérica'.format(parametro))  # Detalle del error
         msg.exec_() # No se bien que hace pero es necesario                 
